<?php

namespace Moblyze\Plan\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Plan\Http\Requests\CrewRequest;
use Moblyze\Plan\Interfaces\CrewRepositoryInterface;
use Moblyze\Plan\Models\Crew;

/**
 * Resource controller class for crew.
 */
class CrewResourceController extends BaseController
{

    /**
     * Initialize crew resource controller.
     *
     * @param type CrewRepositoryInterface $crew
     *
     * @return null
     */
    public function __construct(CrewRepositoryInterface $crew)
    {
        parent::__construct();
        $this->repository = $crew;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Plan\Repositories\Criteria\CrewResourceCriteria::class);
    }

    /**
     * Display a list of crew.
     *
     * @return Response
     */
    public function index(CrewRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Plan\Repositories\Presenter\CrewPresenter::class)
                ->$function();
        }

        $crews = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('plan::crew.names'))
            ->view('plan::crew.index', true)
            ->data(compact('crews', 'view'))
            ->output();
    }

    /**
     * Display crew.
     *
     * @param Request $request
     * @param Model   $crew
     *
     * @return Response
     */
    public function show(CrewRequest $request, Crew $crew)
    {

        if ($crew->exists) {
            $view = 'plan::crew.show';
        } else {
            $view = 'plan::crew.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('plan::crew.name'))
            ->data(compact('crew'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new crew.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CrewRequest $request)
    {

        $crew = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('plan::crew.name')) 
            ->view('plan::crew.create', true) 
            ->data(compact('crew'))
            ->output();
    }

    /**
     * Create new crew.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CrewRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $crew                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plan::crew.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/crew/' . $crew->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plan/crew'))
                ->redirect();
        }

    }

    /**
     * Show crew for editing.
     *
     * @param Request $request
     * @param Model   $crew
     *
     * @return Response
     */
    public function edit(CrewRequest $request, Crew $crew)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('plan::crew.name'))
            ->view('plan::crew.edit', true)
            ->data(compact('crew'))
            ->output();
    }

    /**
     * Update the crew.
     *
     * @param Request $request
     * @param Model   $crew
     *
     * @return Response
     */
    public function update(CrewRequest $request, Crew $crew)
    {
        try {
            $attributes = $request->all();

            $crew->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plan::crew.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/crew/' . $crew->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/crew/' . $crew->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the crew.
     *
     * @param Model   $crew
     *
     * @return Response
     */
    public function destroy(CrewRequest $request, Crew $crew)
    {
        try {

            $crew->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::crew.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('plan/crew/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/crew/' . $crew->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple crew.
     *
     * @param Model   $crew
     *
     * @return Response
     */
    public function delete(CrewRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::crew.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('plan/crew'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/crew'))
                ->redirect();
        }

    }

    /**
     * Restore deleted crews.
     *
     * @param Model   $crew
     *
     * @return Response
     */
    public function restore(CrewRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('plan::crew.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/plan/crew'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/crew/'))
                ->redirect();
        }

    }

}
