<?php

namespace Moblyze\Plan\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Plan\Http\Requests\FloatpassengerRequest;
use Moblyze\Plan\Interfaces\FloatpassengerRepositoryInterface;
use Moblyze\Plan\Models\Floatpassenger;

/**
 * Resource controller class for floatpassenger.
 */
class FloatpassengerResourceController extends BaseController
{

    /**
     * Initialize floatpassenger resource controller.
     *
     * @param type FloatpassengerRepositoryInterface $floatpassenger
     *
     * @return null
     */
    public function __construct(FloatpassengerRepositoryInterface $floatpassenger)
    {
        parent::__construct();
        $this->repository = $floatpassenger;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Plan\Repositories\Criteria\FloatpassengerResourceCriteria::class);
    }

    /**
     * Display a list of floatpassenger.
     *
     * @return Response
     */
    public function index(FloatpassengerRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Plan\Repositories\Presenter\FloatpassengerPresenter::class)
                ->$function();
        }

        $floatpassengers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('plan::floatpassenger.names'))
            ->view('plan::floatpassenger.index', true)
            ->data(compact('floatpassengers', 'view'))
            ->output();
    }

    /**
     * Display floatpassenger.
     *
     * @param Request $request
     * @param Model   $floatpassenger
     *
     * @return Response
     */
    public function show(FloatpassengerRequest $request, Floatpassenger $floatpassenger)
    {

        if ($floatpassenger->exists) {
            $view = 'plan::floatpassenger.show';
        } else {
            $view = 'plan::floatpassenger.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('plan::floatpassenger.name'))
            ->data(compact('floatpassenger'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new floatpassenger.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(FloatpassengerRequest $request)
    {

        $floatpassenger = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('plan::floatpassenger.name')) 
            ->view('plan::floatpassenger.create', true) 
            ->data(compact('floatpassenger'))
            ->output();
    }

    /**
     * Create new floatpassenger.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(FloatpassengerRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $floatpassenger                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plan::floatpassenger.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/floatpassenger/' . $floatpassenger->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plan/floatpassenger'))
                ->redirect();
        }

    }

    /**
     * Show floatpassenger for editing.
     *
     * @param Request $request
     * @param Model   $floatpassenger
     *
     * @return Response
     */
    public function edit(FloatpassengerRequest $request, Floatpassenger $floatpassenger)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('plan::floatpassenger.name'))
            ->view('plan::floatpassenger.edit', true)
            ->data(compact('floatpassenger'))
            ->output();
    }

    /**
     * Update the floatpassenger.
     *
     * @param Request $request
     * @param Model   $floatpassenger
     *
     * @return Response
     */
    public function update(FloatpassengerRequest $request, Floatpassenger $floatpassenger)
    {
        try {
            $attributes = $request->all();

            $floatpassenger->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plan::floatpassenger.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/floatpassenger/' . $floatpassenger->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/floatpassenger/' . $floatpassenger->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the floatpassenger.
     *
     * @param Model   $floatpassenger
     *
     * @return Response
     */
    public function destroy(FloatpassengerRequest $request, Floatpassenger $floatpassenger)
    {
        try {

            $floatpassenger->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::floatpassenger.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('plan/floatpassenger/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/floatpassenger/' . $floatpassenger->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple floatpassenger.
     *
     * @param Model   $floatpassenger
     *
     * @return Response
     */
    public function delete(FloatpassengerRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::floatpassenger.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('plan/floatpassenger'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/floatpassenger'))
                ->redirect();
        }

    }

    /**
     * Restore deleted floatpassengers.
     *
     * @param Model   $floatpassenger
     *
     * @return Response
     */
    public function restore(FloatpassengerRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('plan::floatpassenger.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/plan/floatpassenger'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/floatpassenger/'))
                ->redirect();
        }

    }

}
