<?php

namespace Moblyze\Plan\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Moblyze\Plan\Http\Requests\ManagerRequest;
use Moblyze\Plan\Interfaces\ManagerRepositoryInterface;
use Moblyze\Plan\Models\Manager;

/**
 * Resource controller class for manager.
 */
class ManagerResourceController extends BaseController
{

    /**
     * Initialize manager resource controller.
     *
     * @param type ManagerRepositoryInterface $manager
     *
     * @return null
     */
    public function __construct(ManagerRepositoryInterface $manager)
    {
        parent::__construct();
        $this->repository = $manager;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Moblyze\Plan\Repositories\Criteria\ManagerResourceCriteria::class);
    }

    /**
     * Display a list of manager.
     *
     * @return Response
     */
    public function index(ManagerRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Moblyze\Plan\Repositories\Presenter\ManagerPresenter::class)
                ->$function();
        }

        $managers = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('plan::manager.names'))
            ->view('plan::manager.index', true)
            ->data(compact('managers', 'view'))
            ->output();
    }

    /**
     * Display manager.
     *
     * @param Request $request
     * @param Model   $manager
     *
     * @return Response
     */
    public function show(ManagerRequest $request, Manager $manager)
    {

        if ($manager->exists) {
            $view = 'plan::manager.show';
        } else {
            $view = 'plan::manager.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('plan::manager.name'))
            ->data(compact('manager'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new manager.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ManagerRequest $request)
    {

        $manager = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('plan::manager.name')) 
            ->view('plan::manager.create', true) 
            ->data(compact('manager'))
            ->output();
    }

    /**
     * Create new manager.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ManagerRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $manager                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('plan::manager.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/manager/' . $manager->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/plan/manager'))
                ->redirect();
        }

    }

    /**
     * Show manager for editing.
     *
     * @param Request $request
     * @param Model   $manager
     *
     * @return Response
     */
    public function edit(ManagerRequest $request, Manager $manager)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('plan::manager.name'))
            ->view('plan::manager.edit', true)
            ->data(compact('manager'))
            ->output();
    }

    /**
     * Update the manager.
     *
     * @param Request $request
     * @param Model   $manager
     *
     * @return Response
     */
    public function update(ManagerRequest $request, Manager $manager)
    {
        try {
            $attributes = $request->all();

            $manager->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('plan::manager.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('plan/manager/' . $manager->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/manager/' . $manager->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the manager.
     *
     * @param Model   $manager
     *
     * @return Response
     */
    public function destroy(ManagerRequest $request, Manager $manager)
    {
        try {

            $manager->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::manager.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('plan/manager/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('plan/manager/' . $manager->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple manager.
     *
     * @param Model   $manager
     *
     * @return Response
     */
    public function delete(ManagerRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('plan::manager.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('plan/manager'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/manager'))
                ->redirect();
        }

    }

    /**
     * Restore deleted managers.
     *
     * @param Model   $manager
     *
     * @return Response
     */
    public function restore(ManagerRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('plan::manager.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/plan/manager'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/plan/manager/'))
                ->redirect();
        }

    }

}
